'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateDependenciesVersions = exports.loadDependenciesForComponent = undefined;

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

/**
 * Merge the dependencies-trees we got from all files to one big dependency-tree
 * @param {Array<Object>} depTrees
 * @return {{missing: {packages: Array, files: Array}, tree: {}}}
 */
// function mergeDependencyTrees(depTrees: Array<Object>): Object {
//   const dependencyTree = {
//     missing: { packages: [], files: [], bits: [] },
//     tree: {}
//   };
//   depTrees.forEach((dep) => {
//     if (dep.missing.packages.length) {
//       dependencyTree.missing.packages.push(...dep.missing.packages);
//     }
//     if (dep.missing.files && dep.missing.files.length) {
//       dependencyTree.missing.files.push(...dep.missing.files);
//     }
//     if (dep.missing.bits) {
//       dependencyTree.missing.bits.push(...dep.missing.bits);
//     }
//     Object.assign(dependencyTree.tree, dep.tree);
//   });
//   dependencyTree.missing.packages = R.uniq(dependencyTree.missing.packages);
//   dependencyTree.missing.files = R.uniq(dependencyTree.missing.files);
//   dependencyTree.missing.bits = R.uniq(dependencyTree.missing.bits);
//   return dependencyTree;
// }

/**
 * Load components and packages dependencies for a component. The process is as follows:
 * 1) Use the language driver to parse the component files and find for each file its dependencies.
 * 2) The results we get from the driver per file tells us what are the files and packages that depend on our file.
 * and also whether there are missing packages and files.
 * 3) Using the information from the driver, we go over each one of the dependencies files and find its counterpart
 * component. The way how we find it, is by using the bit.map file which has a mapping between the component name and
 * the file paths.
 * 4) If we find a component to the file dependency, we add it to component.dependencies. Otherwise, it's added to
 * component.missingDependencies.untrackedDependencies
 * 5) Similarly, when we find the packages dependencies, they are added to component.packageDependencies. Otherwise,
 * they're added to component.missingDependencies.missingPackagesDependenciesOnFs
 * 6) In case the driver found a file dependency that is not on the file-system, we add that file to
 * component.missingDependencies.missingDependenciesOnFs
 */
var loadDependenciesForComponent = exports.loadDependenciesForComponent = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee2(component, bitDir, consumer, idWithConcreteVersionString) {
    var _this = this;

    var driver, consumerPath, componentMap, componentFromModel, missingDependencies, _componentMap$getFile, allFiles, testsFiles, getDependenciesTree, dependenciesTree, missingDependenciesOnFs, missingPackagesDependenciesOnFs, missingComponents, missingLinks, componentFiles, traversedDeps, traversedCompDeps, traversedCompDevDeps, dependencies, devDependencies, untrackedDependencies;

    return (_regenerator || _load_regenerator()).default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            driver = consumer.driver;
            consumerPath = consumer.getPath();
            componentMap = component.componentMap;
            componentFromModel = component.componentFromModel;
            missingDependencies = {};
            _componentMap$getFile = componentMap.getFilesGroupedByBeingTests(), allFiles = _componentMap$getFile.allFiles, testsFiles = _componentMap$getFile.testsFiles;

            getDependenciesTree = function () {
              var _ref2 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee() {
                return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        return _context.abrupt('return', driver.getDependencyTree(bitDir, consumerPath, allFiles, component.bindingPrefix));

                      case 1:
                      case 'end':
                        return _context.stop();
                    }
                  }
                }, _callee, _this);
              }));

              return function getDependenciesTree() {
                return _ref2.apply(this, arguments);
              };
            }();
            // find the dependencies (internal files and packages) through automatic dependency resolution


            _context2.next = 9;
            return getDependenciesTree();

          case 9:
            dependenciesTree = _context2.sent;
            missingDependenciesOnFs = {};
            missingPackagesDependenciesOnFs = {};
            missingComponents = {};
            missingLinks = {};
            componentFiles = component.files;

            dependenciesTree.missing.forEach(function (fileDep) {
              var doesFileExistInComponent = !(_ramda || _load_ramda()).default.isEmpty(componentFiles.filter(function (componentFile) {
                return componentFile.relative === fileDep.originFile;
              }));
              if (doesFileExistInComponent) {
                if (fileDep.files && !(_ramda || _load_ramda()).default.isEmpty(fileDep.files)) {
                  if (missingDependenciesOnFs[fileDep.originFile]) {
                    missingDependenciesOnFs[fileDep.originFile].concat(fileDep.files);
                  } else missingDependenciesOnFs[fileDep.originFile] = fileDep.files;
                }
                if (fileDep.packages && !(_ramda || _load_ramda()).default.isEmpty(fileDep.packages)) {
                  // missingDependencies.missingPackagesDependenciesOnFs = fileDep.packages;
                  if (missingPackagesDependenciesOnFs[fileDep.originFile]) {
                    missingPackagesDependenciesOnFs[fileDep.originFile].concat(fileDep.packages);
                  } else missingPackagesDependenciesOnFs[fileDep.originFile] = fileDep.packages;
                }

                if (fileDep.bits && !(_ramda || _load_ramda()).default.isEmpty(fileDep.bits)) {
                  fileDep.bits.forEach(function (missingBit) {
                    var componentId = (_consumer || _load_consumer()).Consumer.getComponentIdFromNodeModulesPath(missingBit, component.bindingPrefix);
                    // todo: a component might be on bit.map but not on the FS, yet, it's not about missing links.
                    if (consumer.bitMap.getExistingComponentId(componentId)) {
                      if (missingLinks[fileDep.originFile]) missingLinks[fileDep.originFile].push(componentId);else missingLinks[fileDep.originFile] = [componentId];
                    } else if (missingComponents[fileDep.originFile]) missingComponents[fileDep.originFile].push(componentId);else missingComponents[fileDep.originFile] = [componentId];
                  });
                }
              }
            });
            if (!(_ramda || _load_ramda()).default.isEmpty(missingDependenciesOnFs)) missingDependencies.missingDependenciesOnFs = missingDependenciesOnFs;
            if (!(_ramda || _load_ramda()).default.isEmpty(missingPackagesDependenciesOnFs)) {
              missingDependencies.missingPackagesDependenciesOnFs = missingPackagesDependenciesOnFs;
            }

            // we have the files dependencies, these files should be components that are registered in bit.map. Otherwise,
            // they are referred as "untracked components" and the user should add them later on in order to commit
            traversedDeps = findComponentsOfDepsFiles(dependenciesTree.tree, allFiles, testsFiles, idWithConcreteVersionString, consumer, component.bindingPrefix, componentFromModel, driver);
            traversedCompDeps = traversedDeps.componentsDeps;
            traversedCompDevDeps = traversedDeps.devComponentsDeps;
            dependencies = Object.keys(traversedCompDeps).map(function (depId) {
              return { id: (_bitId || _load_bitId()).BitId.parse(depId), relativePaths: traversedCompDeps[depId] };
            });

            component.setDependencies(dependencies);
            devDependencies = Object.keys(traversedCompDevDeps).map(function (depId) {
              return { id: (_bitId || _load_bitId()).BitId.parse(depId), relativePaths: traversedCompDevDeps[depId] };
            });

            component.setDevDependencies(devDependencies);
            untrackedDependencies = traversedDeps.untrackedDeps;

            if (!(_ramda || _load_ramda()).default.isEmpty(untrackedDependencies)) missingDependencies.untrackedDependencies = untrackedDependencies;
            component.packageDependencies = traversedDeps.packagesDeps;
            component.devPackageDependencies = traversedDeps.devPackagesDeps;
            component.peerPackageDependencies = findPeerDependencies(consumerPath, component);
            if (!(_ramda || _load_ramda()).default.isEmpty(traversedDeps.relativeDeps)) {
              missingDependencies.relativeComponents = traversedDeps.relativeDeps;
            }
            /*  if (!R.isEmpty(traversedDeps.missingDeps)) {
              if (missingComponents[fileDep.originFile]) missingComponents[fileDep.originFile].push(componentId);
              else missingComponents[fileDep.originFile] = [componentId];
              missingComponents = missingComponents.concat(traversedDeps.missingDeps);
            } */
            if (!(_ramda || _load_ramda()).default.isEmpty(missingLinks)) missingDependencies.missingLinks = missingLinks;
            if (!(_ramda || _load_ramda()).default.isEmpty(missingComponents)) missingDependencies.missingComponents = missingComponents;
            // assign missingDependencies to component only when it has data.
            // Otherwise, when it's empty, component.missingDependencies will be an empty object ({}), and for some weird reason,
            // Ramda.isEmpty returns false when the component is received after async/await of Array.map.
            if (!(_ramda || _load_ramda()).default.isEmpty(missingDependencies)) component.missingDependencies = missingDependencies;

            return _context2.abrupt('return', component);

          case 35:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function loadDependenciesForComponent(_x6, _x7, _x8, _x9) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * The dependency version is determined by the following strategies by this order.
 * 1) if bit.json is different than the model, use bit.json
 * 2) if package.json is different than the model, use package.json. to find the package.json follow this steps:
 * 2 a) search in the component directory for package.json and look for dependencies or devDependencies with the name of the dependency
 * 2 b) if not found there, propagate until you reach the consumer root directory.
 * 2 c) if not found, go directly to the dependency directory and find the version in its package.json
 * 3) if bitmap has a version, use it.
 * 4) use the model if it has a version
 * 5) use the version from bit.json (regardless the status of the model)
 * 6) use the version from package.json (regardless the status of the model)
 *
 * cases where dependency version may be different than the model:
 * 1) user changed bit.json
 * 2) user changed package.json, either, manually or by npm-install —save.
 * 3) user updated a dependency with npm without —save.
 * 4) user imported the dependency with different version causing the bitmap to change.
 */
var updateDependenciesVersions = exports.updateDependenciesVersions = function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee4(consumer, component) {
    var _this2 = this;

    var updateDependencies;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            updateDependencies = function () {
              var _ref4 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee3(dependencies) {
                return (_regenerator || _load_regenerator()).default.wrap(function _callee3$(_context3) {
                  while (1) {
                    switch (_context3.prev = _context3.next) {
                      case 0:
                        dependencies.get().forEach(function (dependency) {
                          var id = dependency.id;
                          var idFromModel = getIdFromModelDeps(component.componentFromModel, id);
                          var idFromBitJson = getIdFromBitJson(component.bitJson, id);
                          var idFromPackageJson = getIdFromPackageJson(consumer, component, id);
                          var idFromBitMap = getIdFromBitMap(consumer, component, id);

                          // get from bitJson when it was changed from the model or when there is no model.
                          var getFromBitJsonIfChanged = function getFromBitJsonIfChanged() {
                            if (!idFromBitJson) return null;
                            if (!idFromModel) return idFromBitJson;
                            if (idFromBitJson !== idFromModel) return idFromBitJson;
                            return null;
                          };
                          // get from packageJson when it was changed from the model or when there is no model.
                          var getFromPackageJsonIfChanged = function getFromPackageJsonIfChanged() {
                            if (!idFromPackageJson) return null;
                            if (!idFromModel) return idFromPackageJson;
                            if (idFromPackageJson !== idFromModel) return idFromPackageJson;
                            return null;
                          };
                          var getFromBitMap = function getFromBitMap() {
                            if (idFromBitMap) return idFromBitMap;
                            return null;
                          };
                          var getFromModel = function getFromModel() {
                            if (idFromModel) return idFromModel;
                            return null;
                          };
                          var getFromBitJsonOrPackageJson = function getFromBitJsonOrPackageJson() {
                            if (idFromBitJson) return idFromBitJson;
                            if (idFromPackageJson) return idFromPackageJson;
                            return null;
                          };
                          var strategies = [getFromBitJsonIfChanged, getFromPackageJsonIfChanged, getFromBitMap, getFromModel, getFromBitJsonOrPackageJson];

                          var _iteratorNormalCompletion4 = true;
                          var _didIteratorError4 = false;
                          var _iteratorError4 = undefined;

                          try {
                            for (var _iterator4 = strategies[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
                              var strategy = _step4.value;

                              var strategyId = strategy();
                              if (strategyId) {
                                dependency.id.version = (_bitId || _load_bitId()).BitId.parse(strategyId).version;
                                (_logger || _load_logger()).default.debug(`found dependency version ${dependency.id.toString()} in strategy ${strategy.name}`);
                                return;
                              }
                            }
                          } catch (err) {
                            _didIteratorError4 = true;
                            _iteratorError4 = err;
                          } finally {
                            try {
                              if (!_iteratorNormalCompletion4 && _iterator4.return) {
                                _iterator4.return();
                              }
                            } finally {
                              if (_didIteratorError4) {
                                throw _iteratorError4;
                              }
                            }
                          }
                        });

                      case 1:
                      case 'end':
                        return _context3.stop();
                    }
                  }
                }, _callee3, _this2);
              }));

              return function updateDependencies(_x12) {
                return _ref4.apply(this, arguments);
              };
            }();

            updateDependencies(component.dependencies);
            updateDependencies(component.devDependencies);

          case 3:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));

  return function updateDependenciesVersions(_x10, _x11) {
    return _ref3.apply(this, arguments);
  };
}();

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _fsExtra;

function _load_fsExtra() {
  return _fsExtra = _interopRequireDefault(require('fs-extra'));
}

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _semver;

function _load_semver() {
  return _semver = _interopRequireDefault(require('semver'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

var _componentMap;

function _load_componentMap() {
  return _componentMap = _interopRequireDefault(require('../bit-map/component-map'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../bit-id');
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('../component'));
}

var _driver;

function _load_driver() {
  return _driver = require('../../driver');
}

var _utils;

function _load_utils() {
  return _utils = require('../../utils');
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../../logger/logger'));
}

var _consumer;

function _load_consumer() {
  return _consumer = require('../../consumer');
}

var _bitJson;

function _load_bitJson() {
  return _bitJson = _interopRequireDefault(require('../bit-json'));
}

var _dependencies;

function _load_dependencies() {
  return _dependencies = _interopRequireDefault(require('./dependencies/dependencies'));
}

var _generalError;

function _load_generalError() {
  return _generalError = _interopRequireDefault(require('../../error/general-error'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/**
 * Given the tree of file dependencies from the driver, find the components of these files.
 * Each dependency file has a path, use bit.map to search for the component name by that path.
 * If the component is found, add it to "componentsDeps". Otherwise, add it to "untrackedDeps".
 *
 * For the found components, add their sourceRelativePath and destinationRelativePath, they are being used for
 * generating links upon import:
 * sourceRelativePath - location of the link file.
 * destinationRelativePath - destination written inside the link file.
 *
 * @param {Object} tree - contains direct deps for each file
 * @param {string[]} files - component files to search
 * @param {string[]} testsFiles - component test files
 * @param {string} entryComponentId - component id for the entry of traversing - used to know which of the files are part of that component
 * @param {consumer} consumer
 * @param {string} bindingPrefix
 * @param componentFromModel
 * @param driver
 */
function findComponentsOfDepsFiles(tree, files, testsFiles, entryComponentId, consumer, bindingPrefix, componentFromModel, driver) {
  var packagesDeps = {};
  var devPackagesDeps = {};
  var componentsDeps = {};
  var devComponentsDeps = {};
  var untrackedDeps = {};
  var relativeDeps = {}; // dependencies that are required with relative path (and should be required using 'bit/').
  var missingDeps = [];

  var consumerPath = consumer.getPath();
  var entryComponentMap = consumer.bitMap.getComponent(entryComponentId);
  var rootDir = entryComponentMap.rootDir;
  var processedFiles = [];

  var traverseTreeForComponentId = function traverseTreeForComponentId(depFile) {
    if (!tree[depFile] || !tree[depFile].files && !tree[depFile].bits) return;
    var rootDirFullPath = (_path || _load_path()).default.join(consumerPath, rootDir);
    if (tree[depFile].files && tree[depFile].files.length) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = tree[depFile].files[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var file = _step.value;

          var fullDepFile = (_path || _load_path()).default.resolve(rootDirFullPath, file);
          var depRelativeToConsumer = (0, (_utils || _load_utils()).pathNormalizeToLinux)((_path || _load_path()).default.relative(consumerPath, fullDepFile));
          var componentId = consumer.bitMap.getComponentIdByPath(depRelativeToConsumer);
          if (componentId) return componentId; // eslint-disable-line consistent-return
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }
    if (tree[depFile].bits && tree[depFile].bits.length) {
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = tree[depFile].bits[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var bit = _step2.value;

          var _componentId = (_consumer || _load_consumer()).Consumer.getComponentIdFromNodeModulesPath(bit, bindingPrefix);
          if (_componentId) return _componentId; // eslint-disable-line consistent-return
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    }

    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
      for (var _iterator3 = tree[depFile].files[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
        var _file = _step3.value;

        if (_file !== depFile) {
          var _componentId2 = traverseTreeForComponentId(_file);
          if (_componentId2) return _componentId2; // eslint-disable-line consistent-return
        } else {
          (_logger || _load_logger()).default.warn(`traverseTreeForComponentId found a cyclic dependency. ${_file} depends on itself`);
        }
      }
    } catch (err) {
      _didIteratorError3 = true;
      _iteratorError3 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion3 && _iterator3.return) {
          _iterator3.return();
        }
      } finally {
        if (_didIteratorError3) {
          throw _iteratorError3;
        }
      }
    }
  };

  var getComponentIdByDepFile = function getComponentIdByDepFile(depFile) {
    var depFileRelative = depFile; // dependency file path relative to consumer root
    var componentId = void 0;
    var destination = void 0;

    if (rootDir) {
      // The depFileRelative is relative to rootDir, change it to be relative to current consumer.
      // We can't use path.resolve(rootDir, fileDep) because this might not work when running
      // bit commands not from root, because resolve take by default the process.cwd
      var rootDirFullPath = (_path || _load_path()).default.join(consumerPath, rootDir);
      var fullDepFile = (_path || _load_path()).default.resolve(rootDirFullPath, depFile);
      depFileRelative = (0, (_utils || _load_utils()).pathNormalizeToLinux)((_path || _load_path()).default.relative(consumerPath, fullDepFile));
    }

    componentId = consumer.bitMap.getComponentIdByPath(depFileRelative);
    // if not found here, the file is not a component file. It might be a bit-auto-generated file.
    // find the component file by the auto-generated file.
    // We make sure also that rootDir is there, otherwise, it's an AUTHORED component, which shouldn't have
    // auto-generated files.
    if (!componentId && rootDir) {
      componentId = traverseTreeForComponentId(depFile);
      if (componentId) {
        // it is verified now that this depFile is an auto-generated file, therefore the sourceRelativePath and the
        // destinationRelativePath should be a partial-path and not full-relative-to-consumer path.
        // since the dep-file is a generated file, it is safe to assume that the componentFromModel has in its
        // dependencies array this component with the relativePaths array. Find the relativePath of this dep-file
        // to get the correct destinationRelativePath. There is no other way to obtain this info.
        if (!componentFromModel) {
          throw new (_generalError || _load_generalError()).default(`Failed to resolve ${componentId} dependencies because the component is not in the model.
Try to run "bit import ${componentId} --objects" to get the component saved in the model`);
        }
        var componentBitId = (_bitId || _load_bitId()).BitId.parse(componentId);
        var dependency = componentFromModel.getAllDependencies().find(function (dep) {
          return dep.id.toStringWithoutVersion() === componentBitId.toStringWithoutVersion();
        });
        if (!dependency) {
          throw new (_generalError || _load_generalError()).default(`the auto-generated file ${depFile} should be connected to ${componentId}, however, it's not part of the model dependencies of ${componentFromModel.id}`);
        }
        var originallySource = entryComponentMap.originallySharedDir ? (0, (_utils || _load_utils()).pathJoinLinux)(entryComponentMap.originallySharedDir, depFile) : depFile;
        var relativePath = dependency.relativePaths.find(function (r) {
          return r.sourceRelativePath === originallySource;
        });
        if (!relativePath) {
          throw new (_generalError || _load_generalError()).default(`unable to find ${originallySource} path in the dependencies relativePaths of ${componentFromModel.id}`);
        }

        componentId = dependency.id.toString();
        destination = relativePath.destinationRelativePath;

        depFileRelative = depFile; // change it back to partial-part, this will be later on the sourceRelativePath
      }
    }

    return { componentId, depFileRelative, destination };
  };

  var processDepFile = function processDepFile(originFile, depFile, importSpecifiers, linkFile) {
    var isTestFile = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

    if (processedFiles.includes(depFile)) return;
    processedFiles.push(depFile);

    var _getComponentIdByDepF = getComponentIdByDepFile(depFile),
        componentId = _getComponentIdByDepF.componentId,
        depFileRelative = _getComponentIdByDepF.depFileRelative,
        destination = _getComponentIdByDepF.destination;
    // the file dependency doesn't have any counterpart component. Add it to untrackedDeps


    if (!componentId) {
      if (untrackedDeps[originFile]) untrackedDeps[originFile].push(depFileRelative);else untrackedDeps[originFile] = [depFileRelative];
      return;
    }

    // happens when in the same component one file requires another one. In this case, there is noting to do
    if (componentId === entryComponentId) return;

    var componentMap = consumer.bitMap.getComponent(componentId);
    // found a dependency component. Add it to componentsDeps
    var depRootDir = componentMap ? componentMap.rootDir : undefined;
    var destinationRelativePath = destination || (depRootDir && depFileRelative.startsWith(depRootDir) ? (0, (_utils || _load_utils()).pathRelativeLinux)(depRootDir, depFileRelative) : depFileRelative);

    var sourceRelativePath = void 0;
    if (linkFile) {
      sourceRelativePath = linkFile;
    } else {
      // when there is no rootDir for the current dependency (it happens when it's AUTHORED), keep the original path
      sourceRelativePath = depRootDir ? depFileRelative : depFile;
    }

    var depsPaths = { sourceRelativePath, destinationRelativePath };
    if (importSpecifiers) {
      depsPaths.importSpecifiers = importSpecifiers;
    }
    var currentComponentsDeps = { [componentId]: [depsPaths] };

    if (componentMap && (componentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.IMPORTED && entryComponentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED || componentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED && entryComponentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.IMPORTED)) {
      // prevent author using relative paths for IMPORTED component
      // also prevent adding AUTHORED component to an IMPORTED component using relative syntax. The reason is that when
      // this component is imported somewhere else, a link-file between the IMPORTED and the AUTHORED must be written
      // outside the component directory, which might override user files.
      if (relativeDeps[originFile]) relativeDeps[originFile].push(componentId);else relativeDeps[originFile] = [componentId];
      return;
    }

    if (componentsDeps[componentId]) {
      // it is another file of an already existing component. Just add the new path
      componentsDeps[componentId].push(depsPaths);
    } else if (devComponentsDeps[componentId]) {
      devComponentsDeps[componentId].push(depsPaths);
    } else if (isTestFile) {
      Object.assign(devComponentsDeps, currentComponentsDeps);
    } else {
      Object.assign(componentsDeps, currentComponentsDeps);
    }
  };

  var processLinkFile = function processLinkFile(originFile, linkFile) {
    var isTestFile = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

    if (!linkFile.dependencies || (_ramda || _load_ramda()).default.isEmpty(linkFile.dependencies)) return;
    var nonLinkFiles = [];
    linkFile.dependencies.forEach(function (dependency) {
      var component = getComponentIdByDepFile(linkFile.file);
      if (component.componentId) {
        // the linkFile is already a component, no need to treat it differently than other depFile
        // aggregate all dependencies using the same linkFile and ultimately run processDepFile
        // with all importSpecifiers of that linkFile.
        // also, delete the linkFile attribute of importSpecifiers so then once the component is
        // imported and the link is generated, it won't be treated as a linkFile.
        dependency.importSpecifiers.map(function (a) {
          return delete a.linkFile;
        });
        nonLinkFiles.push(dependency.importSpecifiers);
      } else {
        processDepFile(originFile, dependency.file, dependency.importSpecifiers, linkFile.file, isTestFile);
      }
    });
    if (nonLinkFiles.length) {
      processDepFile(originFile, linkFile.file, (_ramda || _load_ramda()).default.flatten(nonLinkFiles), undefined, isTestFile);
    }
  };

  var processDepFiles = function processDepFiles(originFile, allDepsFiles, importSpecifiers) {
    var isTestFile = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

    if (allDepsFiles && !(_ramda || _load_ramda()).default.isEmpty(allDepsFiles)) {
      allDepsFiles.forEach(function (depFile) {
        var finalImportSpecifiers = (_ramda || _load_ramda()).default.clone(importSpecifiers);
        if (importSpecifiers) {
          var importSpecifiersFound = importSpecifiers.find(function (specifierFile) {
            return specifierFile.file === depFile;
          });
          if (importSpecifiersFound) finalImportSpecifiers = importSpecifiersFound.importSpecifiers;
        }
        processDepFile(originFile, depFile, finalImportSpecifiers, undefined, isTestFile);
      });
    }
  };

  var processLinkFiles = function processLinkFiles(originFile, allLinkFiles) {
    var isTestFile = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

    if (allLinkFiles && !(_ramda || _load_ramda()).default.isEmpty(allLinkFiles)) {
      allLinkFiles.forEach(function (linkFile) {
        processLinkFile(originFile, linkFile, isTestFile);
      });
    }
  };

  var processBits = function processBits(bits) {
    var isTestFile = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

    if (bits && !(_ramda || _load_ramda()).default.isEmpty(bits)) {
      bits.forEach(function (bitDep) {
        var componentId = (_consumer || _load_consumer()).Consumer.getComponentIdFromNodeModulesPath(bitDep, bindingPrefix);
        var getExistingId = function getExistingId() {
          var existingId = consumer.bitMap.getExistingComponentId(componentId);
          if (existingId) return existingId;

          // maybe the dependencies were imported as npm packages
          if (bitDep.startsWith('node_modules')) {
            // Add the root dir in case it exists (to make sure we search for the dependency package json in the correct place)
            var basePath = rootDir ? (_path || _load_path()).default.join(consumerPath, rootDir) : consumerPath;
            var depPath = (_path || _load_path()).default.join(basePath, bitDep);
            var packageJson = driver.driver.PackageJson.findPackage(depPath);
            if (packageJson) {
              var depVersion = packageJson.version;
              existingId = (_bitId || _load_bitId()).BitId.parse(componentId, depVersion);
              return existingId.toString();
            }
          }
          if (componentFromModel) {
            var modelDep = componentFromModel.getAllDependencies().find(function (dep) {
              return dep.id.toStringWithoutVersion() === componentId;
            });
            if (modelDep) return modelDep.id.toString();
          }
          return null;
        };
        var existingId = getExistingId();
        if (existingId) {
          var currentComponentsDeps = { [existingId]: [] };
          if (!componentsDeps[existingId]) {
            if (isTestFile) {
              Object.assign(devComponentsDeps, currentComponentsDeps);
            } else {
              Object.assign(componentsDeps, currentComponentsDeps);
            }
          }
        } else {
          missingDeps.push(componentId);
        }
      });
    }
  };

  var processPackages = function processPackages(originFile, packages, isTestFile) {
    if (packages && !(_ramda || _load_ramda()).default.isEmpty(packages)) {
      if (isTestFile) {
        Object.assign(devPackagesDeps, packages);
      } else {
        Object.assign(packagesDeps, packages);
      }
    }
  };

  /**
   * Remove the dependencies which appear both in dev and regular deps from the dev
   * Because if a dependency is both dev dependency and regular dependecy it should be treated as regular one
   * Apply for both packages and components dependencies
   */
  var removeDevDepsIfTheyAlsoRegulars = function removeDevDepsIfTheyAlsoRegulars() {
    var devPackagesOnlyNames = (_ramda || _load_ramda()).default.difference((_ramda || _load_ramda()).default.keys(devPackagesDeps), (_ramda || _load_ramda()).default.keys(packagesDeps));
    devPackagesDeps = (_ramda || _load_ramda()).default.pick(devPackagesOnlyNames, devPackagesDeps);
    var devComponentsOnlyNames = (_ramda || _load_ramda()).default.difference((_ramda || _load_ramda()).default.keys(devComponentsDeps), (_ramda || _load_ramda()).default.keys(componentsDeps));
    devComponentsDeps = (_ramda || _load_ramda()).default.pick(devComponentsOnlyNames, devComponentsDeps);
  };

  files.forEach(function (file) {
    var isTestFile = (_ramda || _load_ramda()).default.contains(file, testsFiles);
    processPackages(file, tree[file].packages, isTestFile);
    processBits(tree[file].bits, isTestFile);
    processDepFiles(file, tree[file].files, tree[file].importSpecifiers, isTestFile);
    processLinkFiles(file, tree[file].linkFiles, isTestFile);
  });
  removeDevDepsIfTheyAlsoRegulars();

  return { componentsDeps, devComponentsDeps, packagesDeps, devPackagesDeps, untrackedDeps, relativeDeps, missingDeps };
}

// @todo: move to bit-javascript
/**
 * For author, the peer-dependencies are set in the root package.json
 * For imported components, we don't want to change the peerDependencies of the author, unless
 * we're certain the user intent to do so. Therefore, we ignore the root package.json and look for
 * the package.json in the component's directory.
 */
function findPeerDependencies(consumerPath, component) {
  var componentMap = component.componentMap;
  var getPeerDependencies = function getPeerDependencies() {
    var componentRoot = componentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED ? consumerPath : componentMap.rootDir;
    var packageJsonLocation = (_path || _load_path()).default.join(componentRoot, 'package.json');
    if ((_fsExtra || _load_fsExtra()).default.existsSync(packageJsonLocation)) {
      try {
        var packageJson = (_fsExtra || _load_fsExtra()).default.readJsonSync(packageJsonLocation);
        if (packageJson.peerDependencies) return packageJson.peerDependencies;
      } catch (err) {
        (_logger || _load_logger()).default.error(`Failed reading the project package.json at ${packageJsonLocation}. Error Message: ${err.message}`);
      }
    }
    if (component.componentFromModel && componentMap.origin !== (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED) {
      return component.componentFromModel.peerPackageDependencies;
    }
    return {};
  };
  var projectPeerDependencies = getPeerDependencies();
  var peerPackages = {};
  if ((_ramda || _load_ramda()).default.isEmpty(projectPeerDependencies)) return {};
  // check whether the peer-dependencies was actually require in the code. if so, remove it from
  // the packages/dev-packages and add it as a peer-package.
  Object.keys(projectPeerDependencies).forEach(function (pkg) {
    ['packageDependencies', 'devPackageDependencies'].forEach(function (field) {
      if (Object.keys(component[field]).includes(pkg)) {
        delete component[field][pkg];
        peerPackages[pkg] = projectPeerDependencies[pkg];
      }
    });
  });
  return peerPackages;
}

function getIdFromModelDeps(componentFromModel, componentId) {
  if (!componentFromModel) return null;
  var id = componentFromModel.getAllDependencies().find(function (dep) {
    return dep.id.toStringWithoutVersion() === componentId.toStringWithoutVersion();
  });
  if (!id) return null;
  return id.id.toString();
}

function getIdFromBitJson(bitJson, componentId) {
  var getVersion = function getVersion() {
    if (!bitJson) return null;
    if (!bitJson.dependencies && !bitJson.devDependencies) return null;
    if (bitJson.dependencies[componentId.toStringWithoutVersion()]) {
      return bitJson.dependencies[componentId.toStringWithoutVersion()];
    }
    if (bitJson.devDependencies) {
      return bitJson.devDependencies[componentId.toStringWithoutVersion()];
    }
    return null;
  };
  var version = getVersion();
  if (!version) return null;
  componentId.version = version;
  return componentId.toString();
}

/**
 * the logic of finding the dependency version in the package.json is mostly done in the driver
 * resolveNodePackage method.
 * it first searches in the dependent package.json and propagate up to the consumer root, if not
 * found it goes to the dependency package.json.
 */
function getIdFromPackageJson(consumer, component, componentId) {
  if (!componentId.scope) return null;
  var rootDir = component.componentMap.rootDir;
  var consumerPath = consumer.getPath();
  var basePath = rootDir ? (_path || _load_path()).default.join(consumerPath, rootDir) : consumerPath;
  var packagePath = (_consumer || _load_consumer()).Consumer.getNodeModulesPathOfComponent(component.bindingPrefix, componentId);
  var packageName = packagePath.replace(`node_modules${(_path || _load_path()).default.sep}`, '');
  var modulePath = consumer.driver.driver.resolveModulePath(packageName, basePath, consumerPath);
  if (!modulePath) return null; // e.g. it's author and wasn't exported yet, so there's no node_modules of that component
  var packageObject = consumer.driver.driver.resolveNodePackage(basePath, modulePath);
  if (!packageObject || (_ramda || _load_ramda()).default.isEmpty(packageObject)) return null;
  var packageId = Object.keys(packageObject)[0];
  var version = packageObject[packageId];
  if (!(_semver || _load_semver()).default.valid(version)) return null; // it's probably a relative path to the component
  componentId.version = version.replace(/[^0-9.]/g, ''); // allow only numbers and dots to get an exact version
  return componentId.toString();
}

function getIdFromBitMap(consumer, component, componentId) {
  var componentMap = component.componentMap;
  if (componentMap.dependencies && !(_ramda || _load_ramda()).default.isEmpty(componentMap.dependencies)) {
    var dependencyId = componentMap.dependencies.find(function (dependency) {
      return (_bitId || _load_bitId()).BitId.getStringWithoutVersion(dependency) === componentId.toStringWithoutVersion();
    });
    if (dependencyId) return dependencyId;
  }
  return consumer.bitMap.getExistingComponentId(componentId.toStringWithoutVersion());
}