'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../logger/logger'));
}

var _logger2;

function _load_logger2() {
  return _logger2 = require('../logger/logger');
}

var _scope;

function _load_scope() {
  return _scope = require('../scope');
}

var _exceptions;

function _load_exceptions() {
  return _exceptions = require('../scope/exceptions');
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../bit-id');
}

var _extensionNameNotValid;

function _load_extensionNameNotValid() {
  return _extensionNameNotValid = _interopRequireDefault(require('./exceptions/extension-name-not-valid'));
}

var _analytics;

function _load_analytics() {
  return _analytics = require('../analytics/analytics');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var CORE_EXTENSIONS_PATH = './core-extensions';

// export type BaseExtensionProps = {
//   ...InstanceSpecificProps,
//   ...StaticProps
// };

// type staticProps = $Diff<BaseExtensionProps, instanceSpecificProps>

var BaseExtension = function () {
  function BaseExtension(extensionProps) {
    _classCallCheck(this, BaseExtension);

    this.api = _getConcreteBaseAPI({ name: this.name });

    this.name = extensionProps.name;
    this.rawConfig = extensionProps.rawConfig;
    this.options = extensionProps.options;
    this.dynamicConfig = extensionProps.dynamicConfig || extensionProps.rawConfig;
    this.context = extensionProps.context;
    this.script = extensionProps.script;
    this.disabled = extensionProps.disabled;
    this.filePath = extensionProps.filePath;
    this.loaded = extensionProps.loaded;
    this.api = extensionProps.api;
  }

  /**
   * Run the extension's init function
   */
  // Store the required plugin


  _createClass(BaseExtension, [{
    key: 'init',
    value: function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee() {
        return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                (_analytics || _load_analytics()).Analytics.addBreadCrumb('base-extension', 'initialize extension');
                _context.prev = 1;

                if (!(this.script && this.script.init && typeof this.script.init === 'function')) {
                  _context.next = 5;
                  break;
                }

                _context.next = 5;
                return this.script.init({ rawConfig: this.rawConfig, dynamicConfig: this.dynamicConfig, api: this.api });

              case 5:
                this.initialized = true;
                // Make sure to not kill the process if an extension didn't load correctly
                _context.next = 14;
                break;

              case 8:
                _context.prev = 8;
                _context.t0 = _context['catch'](1);

                (_logger || _load_logger()).default.error(`initialized extension ${this.name} failed`);
                (_logger || _load_logger()).default.error(_context.t0);
                this.initialized = false;
                return _context.abrupt('return', false);

              case 14:
                return _context.abrupt('return', true);

              case 15:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this, [[1, 8]]);
      }));

      function init() {
        return _ref.apply(this, arguments);
      }

      return init;
    }()
  }, {
    key: 'extendAPI',
    value: function extendAPI(baseApi, api) {
      this.api = (_ramda || _load_ramda()).default.merge(baseApi, api);
    }
  }, {
    key: 'toString',
    value: function toString() {
      return JSON.stringify(this, null, 2);
    }
  }, {
    key: 'toBitJsonObject',
    value: function toBitJsonObject() {
      return {
        [this.name]: {
          rawConfig: this.rawConfig,
          options: this.options
        }
      };
    }
  }, {
    key: 'toModelObject',
    value: function toModelObject() {
      return {
        name: this.name,
        config: this.dynamicConfig
      };
    }

    /**
     * Reload the extension, this mainly contain the process of going to the extension file requiring it and get the dynamic config
     * It mostly used for env extension when sometime on the first load the env didn't installed yet (only during build / test) phase
     */

  }, {
    key: 'reload',
    value: function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee2() {
        var baseProps;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                (_analytics || _load_analytics()).Analytics.addBreadCrumb('base-extension', 'reload extension');
                _context2.next = 3;
                return BaseExtension.loadFromFile(this.name, this.filePath, this.rawConfig, this.options);

              case 3:
                baseProps = _context2.sent;

                if (baseProps.loaded) {
                  this.loaded = baseProps.loaded;
                  this.script = baseProps.script;
                  this.dynamicConfig = baseProps.dynamicConfig;
                }

              case 5:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function reload() {
        return _ref2.apply(this, arguments);
      }

      return reload;
    }()
  }, {
    key: 'setExtensionPathInScope',
    value: function setExtensionPathInScope(scopePath) {
      var componentPath = _getExtensionPath(this.name, scopePath, this.options.core);
      this.filePath = componentPath;
    }
  }], [{
    key: 'transformStringToModelObject',
    value: function transformStringToModelObject(name) {
      return {
        name,
        config: {}
      };
    }

    /**
     * Load extension by name
     * The extension will be from scope by default or from file
     * if there is file(path) in the options
     * The file path is relative to the bit.json of the project or absolute
     * @param {string} name - name of the extension
     * @param {Object} rawConfig - raw config for the extension
     * @param {Object} options - extension options such as - disabled, file, core
     * @param {string} consumerPath - path to the consumer folder (to load the file relatively)
     * @param {string} scopePath - scope which stores the extension code
     */

  }, {
    key: 'load',
    value: function () {
      var _ref4 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee3(_ref3) {
        var name = _ref3.name,
            _ref3$rawConfig = _ref3.rawConfig,
            rawConfig = _ref3$rawConfig === undefined ? {} : _ref3$rawConfig,
            _ref3$options = _ref3.options,
            options = _ref3$options === undefined ? {} : _ref3$options,
            consumerPath = _ref3.consumerPath,
            scopePath = _ref3.scopePath,
            context = _ref3.context;

        var concreteBaseAPI, absPath, _file, _staticExtensionProps, _extensionProps, staticExtensionProps, componentPath, extensionProps;

        return (_regenerator || _load_regenerator()).default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                // logger.info(`loading extension ${name}`);
                // Require extension from _debugFile
                (_analytics || _load_analytics()).Analytics.addBreadCrumb('base-extension', 'load extension');
                concreteBaseAPI = _getConcreteBaseAPI({ name });

                if (!options.file) {
                  _context3.next = 12;
                  break;
                }

                absPath = options.file;
                _file = options.file || '';

                if (!(_path || _load_path()).default.isAbsolute(options.file) && consumerPath) {
                  absPath = (_path || _load_path()).default.resolve(consumerPath, _file);
                }
                _context3.next = 8;
                return BaseExtension.loadFromFile(name, absPath, rawConfig, options);

              case 8:
                _staticExtensionProps = _context3.sent;
                _extensionProps = _extends({ api: concreteBaseAPI, context }, _staticExtensionProps);

                _extensionProps.api = concreteBaseAPI;
                return _context3.abrupt('return', _extensionProps);

              case 12:
                staticExtensionProps = {
                  name,
                  rawConfig,
                  dynamicConfig: rawConfig,
                  options,
                  disabled: false,
                  loaded: false,
                  filePath: ''
                };
                // Require extension from scope

                if (!scopePath) {
                  _context3.next = 18;
                  break;
                }

                // $FlowFixMe
                componentPath = _getExtensionPath(name, scopePath, options.core);
                _context3.next = 17;
                return BaseExtension.loadFromFile(name, componentPath, rawConfig, options);

              case 17:
                staticExtensionProps = _context3.sent;

              case 18:
                extensionProps = _extends({ api: concreteBaseAPI, context }, staticExtensionProps);
                return _context3.abrupt('return', extensionProps);

              case 20:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function load(_x) {
        return _ref4.apply(this, arguments);
      }

      return load;
    }()
  }, {
    key: 'loadFromModelObject',
    value: function loadFromModelObject(modelObject) {
      (_analytics || _load_analytics()).Analytics.addBreadCrumb('base-extension', 'load extension from model object');
      var staticExtensionProps = void 0;
      if (typeof modelObject === 'string') {
        staticExtensionProps = {
          name: modelObject,
          rawConfig: {},
          dynamicConfig: {},
          options: {},
          disabled: false,
          loaded: false,
          filePath: ''
        };
      } else {
        staticExtensionProps = {
          name: modelObject.name,
          rawConfig: modelObject.config,
          dynamicConfig: modelObject.config,
          options: {},
          disabled: false,
          loaded: false,
          filePath: ''
        };
      }

      var concreteBaseAPI = _getConcreteBaseAPI({ name: staticExtensionProps.name });
      var extensionProps = _extends({ api: concreteBaseAPI }, staticExtensionProps);
      return extensionProps;
    }
  }, {
    key: 'loadFromFile',
    value: function () {
      var _ref5 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee4(name, filePath) {
        var rawConfig = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
        var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

        var extensionProps, _script, msg;

        return (_regenerator || _load_regenerator()).default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                (_logger || _load_logger()).default.info(`loading extension ${name} from ${filePath}`);
                (_analytics || _load_analytics()).Analytics.addBreadCrumb('base-extension', 'load extension from file');
                extensionProps = {
                  name,
                  rawConfig,
                  dynamicConfig: rawConfig,
                  options,
                  disabled: false,
                  loaded: false,
                  filePath: ''
                };
                // Skip disabled extensions

                if (!options.disabled) {
                  _context4.next = 8;
                  break;
                }

                extensionProps.disabled = true;
                (_logger || _load_logger()).default.info(`skip extension ${extensionProps.name} because it is disabled`);
                extensionProps.loaded = false;
                return _context4.abrupt('return', extensionProps);

              case 8:
                extensionProps.filePath = filePath;
                _context4.prev = 9;

                // $FlowFixMe
                _script = require(filePath); // eslint-disable-line

                extensionProps.script = _script.default ? _script.default : _script;

                if (!(extensionProps.script.getDynamicConfig && typeof extensionProps.script.getDynamicConfig === 'function')) {
                  _context4.next = 16;
                  break;
                }

                _context4.next = 15;
                return extensionProps.script.getDynamicConfig({ rawConfig });

              case 15:
                extensionProps.dynamicConfig = _context4.sent;

              case 16:
                _context4.next = 25;
                break;

              case 18:
                _context4.prev = 18;
                _context4.t0 = _context4['catch'](9);

                if (_context4.t0.code === 'MODULE_NOT_FOUND') {
                  msg = `loading extension ${extensionProps.name} failed, the file ${extensionProps.filePath} not found`;

                  (_logger || _load_logger()).default.warn(msg);
                  // console.error(msg); // eslint-disable-line no-console
                }
                (_logger || _load_logger()).default.error(`loading extension ${extensionProps.name} failed`);
                (_logger || _load_logger()).default.error(_context4.t0);
                extensionProps.loaded = false;
                return _context4.abrupt('return', extensionProps);

              case 25:
                extensionProps.loaded = true;
                return _context4.abrupt('return', extensionProps);

              case 27:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this, [[9, 18]]);
      }));

      function loadFromFile(_x4, _x5) {
        return _ref5.apply(this, arguments);
      }

      return loadFromFile;
    }()
  }]);

  return BaseExtension;
}();

exports.default = BaseExtension;


var _getExtensionPath = function _getExtensionPath(name, scopePath) {
  var isCore = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  if (isCore) {
    return _getCoreExtensionPath(name);
  }
  if (!scopePath) {
    throw new (_exceptions || _load_exceptions()).ScopeNotFound();
  }
  return _getRegularExtensionPath(name, scopePath);
};

var _getCoreExtensionPath = function _getCoreExtensionPath(name) {
  var componentPath = (_path || _load_path()).default.join(__dirname, CORE_EXTENSIONS_PATH, name);
  return componentPath;
};

var _getRegularExtensionPath = function _getRegularExtensionPath(name, scopePath) {
  var bitId = void 0;
  try {
    bitId = (_bitId || _load_bitId()).BitId.parse(name);
  } catch (err) {
    throw new (_extensionNameNotValid || _load_extensionNameNotValid()).default(name);
  }
  if (!bitId || !bitId.scope) throw new (_extensionNameNotValid || _load_extensionNameNotValid()).default(name);

  var internalComponentsPath = (_scope || _load_scope()).Scope.getComponentsRelativePath();
  var internalComponentPath = (_scope || _load_scope()).Scope.getComponentRelativePath(bitId);
  var componentPath = (_path || _load_path()).default.join(scopePath, internalComponentsPath, internalComponentPath);
  try {
    // This might throw an error in case of imported component when the env
    // isn't installed yet
    // It will be handled in higher functions
    var resolved = require.resolve(componentPath);
    return resolved;
  } catch (e) {
    return componentPath;
  }
};

var baseApi = {
  /**
   * API to get logger
   */
  getLogger: function getLogger(name) {
    return function () {
      return (0, (_logger2 || _load_logger2()).createExtensionLogger)(name);
    };
  }
};

/**
 * Function which get actual params and return a concrete base api
 */
var _getConcreteBaseAPI = function _getConcreteBaseAPI(_ref6) {
  var name = _ref6.name;

  var concreteBaseAPI = (_ramda || _load_ramda()).default.clone(baseApi);
  concreteBaseAPI.getLogger = baseApi.getLogger(name);
  return concreteBaseAPI;
};